(in-package :telentweb)

(defclass metacircles-cliki (authed-cliki-handler) ())

(defmethod cliki-default-page-name ((cliki metacircles-cliki))
  "empty docstring"
  "metacircles")

(defmethod cliki:cliki-css-text ((cliki metacircles-cliki) stream)
  (write-sequence
   "
HTML { font-family: times,serif; } 
BODY {  background-color: white ; margin-top: 20; margin-left: 5%; margin-right: 5% }
H1,H2,H3,H4 { font-family: Helvetica,Arial }
H1 {  color: \"00aa55\" }
A.external { color: \"ff00ff\" }
H2 { font-size: 100% }
.sans {  font-family: Helvetica,Arial }
DIV { margin-left: 5%; margin-right: 5% }
" stream))

(defmethod cliki::cliki-page-surround
    ((cliki metacircles-cliki) request function &key title head)
  (let* ((stream (request-stream request))
         (home (cliki:cliki-url-root cliki))
	 (page (find-page cliki title)))
    (html-stream
     stream
     `(html
       (head (title ,(format nil "metacircles ; ~A" title))
	     ,@head
	     ((link :rel "stylesheet" :href
		    ,(urlstring (merge-url home "admin/cliki.css")))))
       (body
	((table :width "100%")
	 ((tr :valign :top)
	  ((td :align left)
	   ((a :href ,(urlstring home))
	    ((img :border 0 :src "images/metacircles-small.png" :alt "[ Home ]"))))
	  ((td ) "[ "((a :href "people") "people") " ]")
	  ((td ) "[ " ((a :href "projects") "projects") " ]")
	  ((td ) "[ " ((a :href "technologies") "technologies") " ]")
	  ((td ) "[ " ((a :href "rates") "rates") " ]")
	  ((td ) "[ " ((a :href "partnerships") "partnerships") " ]")
	  ((td ) "[ " ((a :href "contact") "contact") " ]")
	  ((td ) "[ " ((a :href "further reading") "further reading") " ]")
	  )
	 ((tr)
	  ((td :colspan 8 :align :right)
	   ((form :action "/admin/search" :method get)
	    "Search: "
	    ((input :name words :size 30 :type text))))))
	(h1 ,title)
	,function
	(hr)
	((font :size 2)
	 "Page last updated: "
	 ,(if page
	      (cliki::universal-time-to-rfc-date
	       (file-write-date (cliki::page-pathname page)))
	      "(none)")
	 (br)
	 "Copyright &copy; 2001-2003 metacircles Ltd.  All trademarks are acknowledged as the property of their respective authors.  View "
	 ((a :href "Privacy+Statement") "Privacy Statement")))))))


(defclass survey-handler (handler) ())

(defmethod handle-request-response ((handler survey-handler)
				    (method (eql :get)) request)
  (let ((cliki (request-cliki request)))
    (request-send-headers request)
    (with-page-surround (cliki request "cirCLe CD Feedback")
      (html-stream
       cliki::out
       `(div
	 ((form :method :post)
	  (p "If you have comments about the proposed <a href=\"/circle+cd\">cirCLe CD</a>, or would like to receive further news about it as it happens, please fill this form in.  All sections are optional, except that we need a valid email address if you want to receive email.  If you'd like to provide other or more detailed feedback, please feel free to contact us directly at circle@metacircles.com")
	  (p "1) What features in the cirCLe CD do you particularly want/like/dislike/not care about?")
	  ((textarea :rows 10 :cols 60 :name :text))
	  (p "2) If you'd be interested in purchasing this, please indicate a likely price or price range. "
	     (i "Filling in this form does not commit you to anything; this is solely to gauge likely demand"))
	  (p (b "Price: US$") ((input :type text :length 25 :name price)))
	  (p "3) Your email address (optional, unless you want to receive announcements)")
	  (p (b "Email: ") ((input :type text :length 40 :name email)))
	  (p "4) Would you like to receive more information about the cirCLe CD if we decide to proceed?  This will add you to a "
	     ((a :href "http://lists.metacircles.com/cgi-bin/mailman/listinfo/circle-announce")
	      "low-volume announcements-only mailing list"))
	  (p (b "Subscribe: ") ((input :type checkbox :name list-p :value t)))
	  (p ((input :type :submit :value "Send Feedback")))))))
    t))


(defmethod handle-request-response ((handler survey-handler)
				    (method (eql :post))  request)
  (let* ((body (request-body request))
	 (cliki (request-cliki request))
	 error
	 (address (body-param "EMAIL" body))
	 (text (body-param "TEXT" body))
	 (price (body-param "PRICE" body))
	 (list-p (body-param "LIST-P" body)))
    (request-send-headers request)
    (when list-p
      (if (> (length address) 0)
	  (handler-case
	      (let ((smtp
		     (make-instance 'smtp:mail-output-stream
				    :sender address
				    :recipients
				    (list
				     "dan@telent.net"
				     "circle-announce-request@metacircles.com"))))
		(format smtp "From: ~A
To: circle-announce-request@metacircles.com
Subject: subscribe

Subscription request from ~A" address (request-url request))
		(close smtp))
	    (smtp:mail-error (c)
	      (setf error
		    (format nil "~{~A ~}" (smtp:error-response c)))))
	  (setf error "No email address was supplied")))
    (with-open-file (comments (merge-pathnames "circle.feedback.txt"
					       (user-homedir-pathname))
			      :direction :output
			      :if-does-not-exist :create
			      :if-exists :append)
      (format comments "---------------------------------------
Price: ~A
Name: ~A ~A
Comments: ~A~%" price address (or error "") text))
    (if error
	(with-page-surround (cliki request "Error sending mail")
	  (html-stream
	   cliki::out
	   `(div
	     (p "We're unable to subscribe you to the announcement mailing list: you might want to try again.  Your comments have been logged, though: thanks.")
	     (pre ,(princ-to-string error)))))
	(with-page-surround (cliki request "Thanks for your feedback")
	  (html-stream
	   cliki::out
	   `(div
	     ,@(when list-p
		     `((p "You will shortly receive an email asking you to confirm your address.  You will not receive further email from us until/unless you confirm your subscription by following the instructions in that mail")))
	     (p "Thanks for your feedback, which will help us determine the likely market for this product."))))))
  t)


(defmethod shared-initialize :after ((handler metacircles-cliki)
				     slot-names &rest initargs)
  (install-handler
   handler
   (make-instance 'survey-handler)
   "circle-survey" t)
  (install-handler
   handler (make-instance 'araneida:static-file-handler
			  :pathname (merge-pathnames
				     "images/"
				     (cliki:cliki-data-directory handler)))
   "images/" nil)

  (install-handler
   handler (make-instance 'araneida::static-file-handler
			  :pathname
			  (parse-namestring
			   "/usr/local/web/metacircles-web/htdocs/log/"))
   "log/" nil))
