
(defun system (format-string &rest args)
  (ext:run-program "sh" (list "-c" (apply #'format nil format-string args))
                   :wait t :input nil :output t :error :output ))

(defvar *export-symbols* nil)
(defun c-for-structure (stream lisp-name c-struct)
  (destructuring-bind (c-name &rest elements) c-struct
    (format stream "printf(\"(define-c-struct ~A %d)\\n\",sizeof (~A));~%" lisp-name c-name)
    (dolist (e elements)
      (destructuring-bind (lisp-type lisp-el-name c-type c-el-name) e
        (format stream "printf(\"(define-c-accessor ~A-~A ~A ~A \");~%"
                lisp-name lisp-el-name lisp-name lisp-type)
        ;; offset
        (format stream "{ ~A t;printf(\"%d \",((unsigned long)&(t.~A)) - ((unsigned long)&(t)) ); }~%"
                c-name c-el-name)
        ;; length
        (format stream "{ ~A t;printf(\"%d\",(sizeof t.~A));}~%"
                c-name c-el-name)
        (format stream "printf(\")\\n\");~%")))))

(defun c-for-function (stream lisp-name alien-defn)
  (destructuring-bind (c-name &rest definition) alien-defn
    (let ((*print-right-margin* nil))
      (format stream "printf(\"(declaim (inline ~A))\\n\");~%"
              lisp-name)
      (princ "printf(\"(def-foreign-routine (" stream)
      (princ "\\\"" stream) (princ c-name stream) (princ "\\\" " stream)
      (princ lisp-name stream)
      (princ " ) " stream)
      (dolist (d definition)
        (write d :length 999999999999999999999999 :right-margin 9999999999999999 :stream stream)
        (princ " " stream))
      (format stream ")\\n\");~%"))))


(defun print-c-source (stream headers definitions package-name)
  ;(format stream "#include \"struct.h\"~%")
  (let ((*print-right-margin* nil))
    (loop for i in headers
          do (format stream "#include <~A>~%" i))
    (format stream "main() { ~%
printf(\"(in-package ~D)\\\n\");~%" package-name)  
    (dolist (def definitions)
      (destructuring-bind (type lispname cname &optional doc) def
        (cond ((eq type :integer)
               (format stream
                       "printf(\"(defconstant ~A %d \\\"~A\\\")\\\n\",~A);~%"
                       lispname doc cname))
              ((eq type :string)
               (format stream
                       "printf(\"(defconstant ~A %S \\\"~A\\\")\\\n\",~A);~%"
                     lispname doc cname))
              ((eq type :function)
               (c-for-function stream lispname cname))
              ((eq type :structure)
               (c-for-structure stream lispname cname))
              (t
               (format stream
                       "printf(\";; Non hablo Espagnol, Monsieur~%")))))
    (format stream "}")))

(defun c-constants-extract  (filename output-file package)
  (let* ((directory (subseq filename 0 (position #\/ filename :from-end t) ))
         (pos (position #\/ filename :from-end t))
         (nodename (subseq filename
                           (if pos (1+ pos) 0)))
         (tmpname (format nil "/tmp/~A.~A.c" nodename (unix:unix-getpid))))
    (with-open-file (f tmpname :direction :output)
      (with-open-file (i filename :direction :input)
        (let* ((headers (read i))
               (definitions (read i)))
          (print-c-source  f headers definitions package))))
    (system "cc -o ~A.exe ~A" tmpname tmpname)
    (system "~A.exe >~A" tmpname output-file)
    (with-open-file (f (format nil "~A-package.lisp"
                               (string-downcase (symbol-name package)))
                       :direction :output)
      (print 
       `(defpackage ,package
          (:use "COMMON-LISP" "ALIEN" "SYSTEM" "EXT" "C-CALL")
          (:export ,@*export-symbols*))
       f))))



