(in-package :user)
;;; turn water into wine ^W^W^W lisp into HTML

#|
OK.  We need a design

1) The aim is to document the current package, given a system.
2) We can do it all without loading the system, so don't use the current
image to look stuff up
3) We output HTML on *standard-output*
4) Hyperlink wherever useful
5) We're allowed to intern symbols all over the place if we like

|#

;;; note: break badly on multiple packages


(defvar *symbols* nil
  "List of external symbols to print; derived from parsing DEFPACKAGE form")

(defun do-defpackage (form stream)
  (setf *symbols* nil)
  (destructuring-bind (defn name &rest options) form
    (when (string-equal name (package-name *package*))
      (format stream "<h1>Package ~A</h1>~%" name)
      (let ((exports (assoc :export options)))
        (when exports
          (setf *symbols* (mapcar #'symbol-name (cdr exports)))
          (format stream "<h2>Exports</h2>~{~A ~}"
                  (mapcar
                   (lambda (x) (format nil "<a href=#~A >~A</a> " x x))
                   (sort (cdr exports) #'string< :key #'symbol-name    ))))))))

(defun do-defunlike (form label stream)
  (destructuring-bind (defn name lambdalist doc &rest code) form
    (when (member (symbol-name name) *symbols* :test #'string=)
      (setf *symbols* (remove (symbol-name name) *symbols* :test #'string=))
      (format stream "<hr><p><a name=\"~A\"><i>~A: </i><b>~A</b></a>~%"
              name label name)
      (format stream "<p><b>Syntax:</b>~%<p><b>~A</b> <i>~A</i>~%"
              name lambdalist)
      (if (stringp doc)
          ;; we'd like to replace all-capitalized words with
          ;; <a href="#NAME">NAME</a> 
          ;; (when (member (symbol-name name) *symbols* :test #'string=)
          ;; but presently we don't
          (format stream "<blockquote>~A</blockquote>~%" doc)))))

(defun do-defun (form stream) (do-defunlike form "Function" stream))
(defun do-defmethod (form stream) (do-defunlike form "Method" stream))
(defun do-defgeneric (form stream) (do-defunlike form "Generic Funtion" stream))

(defun do-file (input-stream output-stream)
  "Read in a Lisp program on INPUT-STREAM and make semi-pretty HTML on OUTPUT-STREAM"
  (let ((eof-marker (gensym)))
    (do
        ((form (read input-stream nil eof-marker)
               (read input-stream nil eof-marker)))
        ((eq form eof-marker) nil)
      (cond ((not (listp form)) nil)
            #+nil ((eq (car form) 'defclass)
             (do-defclass form output-stream))
            ((eq (car form) 'defpackage)
             (do-defpackage form output-stream))

            ((eq (car form) 'defun)
             (do-defun form output-stream))
            ((eq (car form) 'defmethod)
             (do-defmethod form output-stream))
            ((eq (car form) 'defgeneric)
             (do-defgeneric form output-stream))

            (t nil)))
    (format output-stream "~%<!--  Formatted at ~A -->~%"
            (format-universal-time nil (get-universal-time) :style :long))))

(defun document-file-operation (component force)
  "This is a defsystem operation.  "
  (let* ((source-file (make::component-pathname component :source))
         (source-ext (make::component-extension component :source))
         (name (format nil "~A.~A" source-file source-ext)))
    (with-open-file (in name :direction :input)
      (do-file in *standard-output*)))
  nil)

(defun document-system (system &key
                               (output-stream *standard-output*)
                               (package *package*))
  "Read the SYSTEM definition and produce HTML documentation for all
symbols exported from PACKAGE."
  (let ((*package* (find-package package))
        (*standard-output* output-stream))
    (make::component-operation 'document 'document-file-operation)
    (oos system 'document)))

(with-open-file (*standard-output* "documentation.html" :direction :output)
  (format t "<html><head><title>:SOCKETS hackydoc</title></head><body>~%")
  (document-system 'sockets :package :sockets))

