(in-package :sockets-internal)

;;;; Foreign function glue.  This is the only file in the distribution
;;;; that's _intended_ to be vendor-specific.  The macros defined here
;;;; are called from constants.lisp, which was generated from constants.ccon
;;;; by the C compiler as driven by that wacky def-to-lisp thing.

;;;; of course, the whole thing is vendor-specific actually, due to
;;;; the way we use cmucl alien types in constants.ccon as a cheap way
;;;; of transforming C-world alues into Lisp-world values.  But if
;;;; anyone were to port that bit to their preferred implementation, they
;;;; wouldn't need to port all the rest of the cmucl alien interface at
;;;; the same time

;;; (def-foreign-routine ("stat" STAT ) (INTEGER 32) (FILE-NAME
;;; C-CALL:C-STRING) (BUF (* T)) )

;;; I can't help thinking this was originally going to do something a
;;; lot more complex
(defmacro def-foreign-routine
  (&whole it (c-name lisp-name) return-type &rest args)
  `(def-alien-routine ,@(cdr it)))

;;; define-c-accessor makes us a setter and a getter for changing
;;; memory at the appropriate offset
;;; (define-c-accessor STAT-ATIME STAT (INTEGER 32) 56 4)
(defmacro define-c-accessor (el structure type offset length)
  (declare (ignore structure length))
  `(progn
     (defun ,el (ptr)
       (deref (sap-alien (ext::sap+ (alien-sap ptr) ,offset) (* ,type))))
     (defun (setf ,el) (newval ptr)
       (setf (deref (sap-alien (ext::sap+ (alien-sap ptr) ,offset) (* ,type)))
             newval))))


;;; make memory allocator for appropriately-sized block of memory, and
;;; a constant to tell us how big it was anyway
(defmacro define-c-struct (name size)
  (labels ((p (x) (intern (concatenate 'string x (symbol-name name)))))
    `(progn
       (defun ,(p "ALLOCATE-") () (sockint::malloc ,size))
       (defconstant ,(p "SIZE-OF-") ,size)
       (defun ,(p "FREE-" ) (p)
         (alien:free-alien p)))))

(defun foreign-nullp (c)
  "C is a pointer to 0?"
  (= 0 (system:sap-int (alien:alien-sap  c))))

;;; this could be a lot faster if I cared enough to think about it
(defun foreign-vector (pointer size length)
  "Compose a vector of the words found in foreign memory starting at
POINTER.  Each word is SIZE bytes long; LENGTH gives the number of
elements of the returned vector.  See also FOREIGN-VECTOR-UNTIL-ZERO"
  (let ((ptr (alien:cast pointer (* (alien:unsigned 8)))))
    (coerce 
     (loop for i from 0 to (1- length) by size
           collect
           (loop for j from 0 to (1- size)
                 for tot = (+ (or tot 0)
                              (* (expt 256 (- (1- size) j))
                                 (alien:deref ptr (+ i j))))
;                 do (format t "~A ~A ~A ~A ~A ~%"
;                            (alien:deref ptr (+ i j))
;                            i j (expt 256 (- (1- size) j)) tot)
                 finally do return tot))
     'vector)))

