(in-package :sockets)
;;; name service

;;; initially, "whatever gethostbyname uses" - which may be /etc/hosts,
;;; NIS, DNS, or something specified by /etc/nsswitch.conf

;;; direct links to resolver(3) async routines would be nice to have
;;; eventually, for doing lookups in parallel with other things

(defclass host-ent ()
  ((name :initarg :name :accessor host-ent-name)
   (aliases :initarg :aliases :accessor host-ent-aliases)
   (type :initarg :type :accessor host-ent-address-type) ; always BAF_INETP?
   (length :initarg :length :accessor host-ent-length) ;h_length - what's this for?
   (addresses :initarg :addresses :accessor host-ent-addresses)))

(defmethod host-ent-address ((host-ent host-ent))
  (car (host-ent-addresses host-ent)))

(define-condition host-not-found-error (socket-error)) ; host unknown
(define-condition no-address-error (socket-error)) ; valid name but no IP address
(define-condition no-recovery-error (socket-error)) ; name server error
(define-condition try-again-error (socket-error)) ; temporary

(defun get-host-by-name (host-name)
  "Returns a HOST-ENT instance for HOST-NAME or throws some kind of condition. HOST-NAME may also be an IP address in dotted quad notation or some other weird stuff - see gethostbyname(3) for grisly details."
  (let* ((h (sockint::gethostbyname host-name))
         (addresses nil)
         (length (and (not (sockint::foreign-nullp h))
                      (sockint::hostent-length h)))
         (aliases nil))
    (if (sockint::foreign-nullp h) (socket-error "gethostbyname"))
    (do ((i 0 (1+ i)))
        ((not (alien:deref (sockint::hostent-aliases h) i)) nil)
      (push (alien:deref (sockint::hostent-aliases h) i) aliases))
    (do* ((i 0 (1+ i))
          (a (alien:deref (sockint::hostent-addresses h) i)
             (alien:deref (sockint::hostent-addresses h) i)))
        ((sockint::foreign-nullp a) nil)
      (push (sockint::foreign-vector a 1 length) addresses))
    (make-instance 'host-ent
                   :name (sockint::hostent-name h)
                   :aliases (reverse aliases)
                   :addresses (reverse addresses))))

;;; if we're using dns, and if we're on a machine that uses multiple
;;; networks with different nameservers on each, we need to call res_init()
;;; when we move from one to the next, because the resolver library only
;;; checks /etc/resolv.conf once per process

(defun dns-res-init ()
  (sockint::res-init))
