(in-package :cliki)

(defun version-open-p (cliki page version user)
  ;; If we're saving the current page version, and the file exists, and
  ;; it was written less than five minutes ago, and the current user is
  ;; the same as was supplied for the previous edit, then return true.
  ;; Otherwise, return false.
  ;;
  ;; Essentially, allow users to submit an edit, look at their changes,
  ;; decide that they don't like it, hit back to get to the edit form
  ;; again, make more changes, and resubmit, so long as they're quick
  ;; about it.
  (and (eql version (Car (page-versions page)))
       (let ((file-date (file-write-date (page-pathname page :version version)))
	     (five-minutes-ago (- (get-universal-time) (* 5 60))))
	 (and file-date
	      (> file-date five-minutes-ago)))
       (destructuring-bind (date title user- descr)
	   (find-recent-change  cliki (page-title page))
	 (declare (ignore date title descr))
	 (string= user user-))))

(defmethod check-page-save-allowed ((cliki cliki-instance) page version user)
  (unless
      (or (not (page-versions page))
	  (version-open-p cliki page version user)
	  (= version (1+ (car (page-versions page)))))
    (signal 'cliki-page-save-rejected
	    :client-message
	    "Simultaneous edit: a newer version of this page already exists")))

(defun can-save-as-version-p (cliki page version user)
  (handler-case 
      (progn (check-page-save-allowed cliki page version user) t)
    (cliki-page-save-rejected () nil)))



