(in-package :entomotomy)

(defun coerce-to-version (designator)
  (typecase designator
    (string (cdr (assoc designator
			'(("0.7.4" . 11)
			  ("0.7.5" . 12)
			  ("18b" . 10))
			:test #'string=)))
    (t designator)))

(defmethod search-term-relevance ((cliki cliki-instance) page
				  (term (eql :patch)) &rest args)
  (let ((search-tree (getf args :tree t)))
    (dolist (patch (cdr (assoc term (cliki::page-indices page))))
      (let ((tree (getf patch :tree nil)))
	(if (or (eql search-tree t) (eql search-tree tree))
	    (return-from search-term-relevance 1))))
    0))



(defmethod compute-index-for-page ((cliki cliki-instance)
				   (page cliki::cliki-page)
				   (index-name (eql :open)) index)
  (let ((terms
	 (loop for c in (cdr (assoc :closed (cliki::page-indices page)))
	       collect `(:open-p nil ,@c))))
    (sort (append index terms)
	  #'<
	  :key (lambda (x) (getf x :version 0)))))

(defvar *trees* '(or :cmucl :sbcl))
(defvar *arches* '(or :x86 :alpha :sparc :ppc :hppa :mips))
(defvar *oses* '(or :linux :freebsd :openbsd :sunos :osf1))
(defmethod search-term-relevance ((cliki cliki-instance) page
				  (term (eql :open)) &rest search-for)
  (let ((reports (cdr (assoc :open (cliki::page-indices page))))
	(search-date (coerce-to-version 
		      (getf search-for :version (get-universal-time)))))
    ;; note that the page indexing methods must
    ;; (1) merge the closed reports into the same list as the
    ;;   open ones
    ;; (2) keep the whole thing in date order
    (let ((bug (make-hash-table :test 'equal))) 
      (dolist (report reports)
	(destructuring-bind
	      (&key 
	       (version 0)
	       (open-p t)
	       (tree (cdr *trees*))	;strip leading 'or
	       (arch (cdr *arches*))
	       (os (cdr *oses*))
	       &allow-other-keys)
	    report
	  (when (<= version search-date)
	    (labels ((listify (c) (if (listp c)  c (list c))))
	      (set-bug-state bug open-p (listify tree)
			     (listify arch) (listify os))))))
      (destructuring-bind (&key (tree *trees*) (arch *arches*) (os *oses*)
				&allow-other-keys) search-for
	(if (openp bug tree arch os) 1 0)))))

(defmethod search-term-relevance ((cliki cliki-instance) page
				  (term (eql :closed)) &rest search-for)
  (- 1 (apply #'search-term-relevance cliki page :open search-for)))





(defgeneric runsp (tree arch os))
(defmethod runsp ((tree (eql :sbcl)) arch os)
  (case os
    (:sunos (eql arch :sparc))
    ((:freebsd :openbsd) (eql arch :x86))
    (:linux t)
    (:osf1 (eql arch :alpha))
    (t nil)))

(defmethod runsp ((tree (eql :cmucl)) arch os)
  (case os
    (:sunos (eql arch :sparc))
    ((:freebsd :openbsd :linux) (eql arch :x86))
    (t nil)))

(defun set-bug-state (bug-state value trees arches oses)
  (dolist (tree trees)
    (dolist (arch arches)
      (dolist (os oses)
	(when (runsp tree arch os)
	  (setf (gethash (list tree arch os) bug-state) value))))))

;;; unused
(defun make-open-bug (&key
		      (tree '(:cmucl :sbcl))
		      (arch '(:x86 :alpha :sparc :ppc :hppa :mips))
		      (os '(:linux :freebsd :openbsd :sunos :osf1)))
  (let ((bug-state (make-hash-table :test 'equal)))
    (set-bug-state bug-state t tree arch os)
    bug-state))

;;; likewise, I think
(defun process-reports (&rest reports)
  (let ((open-request (car reports)))
    (unless (eq (car open-request) 'open)
      (error "Attempting to close a bug before opening it"))
    (let ((bug (apply #'make-open-bug (cdr open-request))))
      (dolist (report (cdr reports))
	(destructuring-bind
	      (request
	       &key 
	       (tree '(:cmucl :sbcl))
	       (arch '(:x86 :alpha :sparc :ppc :hppa :mips))
	       (os '(:linux :freebsd :openbsd :sunos :osf1)))
	    report
	  (case request
	    (open (set-bug-state bug t tree arch os))
	    (close (set-bug-state bug nil tree arch os)))))
      bug)))

(defun openp-1 (bug tree arch os)
  (gethash (list tree arch os) bug))

(defun openp (bug tree-spec arch-spec os-spec)
  (cond
    ((and (atom tree-spec)
	  (atom arch-spec)
	  (atom os-spec))
     (openp-1 bug tree-spec arch-spec os-spec))
    ((and (listp tree-spec) (eq (car tree-spec) 'and))
     (every #'(lambda (tree) (openp bug tree arch-spec os-spec)) (cdr tree-spec)))
    ((and (listp tree-spec) (eq (car tree-spec) 'or))
     (some #'(lambda (tree) (openp bug tree arch-spec os-spec)) (cdr tree-spec)))
    #| Not doesn't mean this, I think
    ((and (listp tree-spec) (eq (car tree-spec) 'not))
     (not (openp bug (cadr tree-spec) arch-spec os-spec)))
    |#
    ((and (listp arch-spec) (eq (car arch-spec) 'and))
     (every #'(lambda (arch) (openp bug tree-spec arch os-spec)) (cdr arch-spec)))
    ((and (listp arch-spec) (eq (car arch-spec) 'or))
     (some #'(lambda (arch) (openp bug tree-spec arch os-spec)) (cdr arch-spec)))
    #|
    ((and (listp arch-spec) (eq (car arch-spec) 'not))
     (not (openp bug tree-spec (cadr arch-spec) os-spec)))
    |#
    ((and (listp os-spec) (eq (car os-spec) 'and))
     (every #'(lambda (os) (openp bug tree-spec arch-spec os)) (cdr os-spec)))
    ((and (listp os-spec) (eq (car os-spec) 'or))
     (some #'(lambda (os) (openp bug tree-spec arch-spec os)) (cdr os-spec)))
    #|
    ((and (listp os-spec) (eq (car os-spec) 'not))
     (not (openp bug tree-spec arch-spec (cadr os-spec))))
    |#
    ))
