(defpackage "SOCKETS-INTERNAL"
  (:nicknames "SOCKINT")
  (:shadow close listen)
  #+cmu (:use "COMMON-LISP" "ALIEN" "SYSTEM" "EXT" "C-CALL")
  #+sbcl (:use "COMMON-LISP" "SB-ALIEN" #+nil "SB-SYSTEM" "SB-EXT" "SB-C-CALL"))

;;; SBCL changes a lot of package prefixes.  To avoid littering the
;;; code with conditionals, we use the SBCL package prefixes
;;; throughout.  This means that we need to create said packages
;;; first, if we're using CMUCL

;;; One thing that this exercise really has made clear is just how much
;;; of the alien stuff is scattered around the cmucl package space
;;; seemingly at random.  Hmm.

#+cmu
(eval-when (:compile-toplevel :load-toplevel)
  (defun add-package-nickname (name nickname)
    (let ((p (find-package name)))
      (rename-package p (package-name p)
                      (cons nickname (package-nicknames name)))))
  (add-package-nickname "EXT" "SB-EXT")
  (add-package-nickname "ALIEN" "SB-ALIEN")
  (add-package-nickname "UNIX" "SB-UNIX")
  (add-package-nickname "C-CALL" "SB-C-CALL")
  (add-package-nickname "SYSTEM" "SB-SYS"))

(defpackage "SOCKETS"
  (:export socket unix-socket inet-socket
           make-unix-socket make-inet-socket
           socket-bind socket-accept socket-connect
           socket-send socket-receive socket-recv
           socket-name socket-peername socket-listen
           socket-close socket-file-descriptor socket-make-stream
           get-protocol-by-name
           get-host-by-name

           host-ent
           host-ent-addresses host-ent-address
           host-ent aliases host-ent-name
           
           sockopt-reuse-address

           bad-file-descriptor-error
           address-in-use-error
           interrupted-error
           bad-file-descriptor-error
           invalid-argument-error
           interrupted-error
           out-of-memory-error
           operation-not-supported-error
           operation-not-permitted-error
           protocol-not-supported-error
           network-unreachable-error
           
           make-inet-address

           non-blocking-mode
           )
  (:use "COMMON-LISP" "SOCKETS-INTERNAL")
  (:documentation
   "
An OO-ish socket API for CMUCL and SBCL.  Ideas shamelessly stolen from
the BSD socket API for C and Graham Barr's IO::Socket classes for Perl

We borrow the OO concept of the Perl interface, but introduce new
methods to get closer to the flexibility of C, and rename a lot of
methods and arguments to fit Lisp style more closely.

Support structures: socket-address (sockaddr), socket-address-inet
(sockaddr_in), socket-address-unix likewise: work as you'd expect
"
))




