(in-package :telentweb)

(defvar *cclan-mirror-list* nil)
(eval-when (:load-toplevel)
  (setf *cclan-mirror-list*
	(merge-pathnames "mirrors.sexp" *load-truename*)))

(defun read-cclan-mirror-list ()
  (with-open-file (i *cclan-mirror-list*) (read i)))

(defun cclan-accept-redirect-cookie (request)
  (let ((cookie (car (url-query-param (request-url request) "M"))))
    (cond (cookie
	   (request-send-headers
	    request
	    :set-cookie (format nil "CCLAN-SITE=~A; path=/; expires=~A; domain=~A"
				cookie (net.telent.date:universal-time-to-rfc-date
					(+ (* 365 86400) (get-universal-time)))
				(url-host (request-url request))))
	   (write-sequence
	    (html
	     `(html
	       (head (title "Set Cookie"))
	       (body
		(h1 "Set Cookie")
		(p "You have selected " ,cookie " as your preferred CCLAN mirror"))))
	    (request-stream request)))
	  (t
	   (let ((mirrors (read-cclan-mirror-list)))
	     (request-send-headers request)
	     (labels ((site (s)
			`(li
			  ((a :href
			    ,(urlstring (merge-url
					 (request-url request)
					 (format nil "?M=~A"
						 (urlstring-escape (car s))))))
			   ,(car s)) ,@(cdr s))))
	       (write-sequence
		(html
		 `(html
		   (head (title "Select Mirror"))
		   (body
		    (h1 "Select Mirror")
		    (p "Before you can use the CCLAN redirector, you need to set up a cookie containing the name of your preferred <a href=\"/cliki/vn-cclan\">vn-cclan</a> mirror site")
		    (p "If you appear unable to get past this page, your browser, proxy, local firewall or ISP probably are not letting our cookie past.  Or possibly the clock on your computer is over a year fast (cookie expires in 365 days)")
		    (ul 
		     ,@(mapcar #'site mirrors)
		     (li ((form :method :get :action "../cclan-choose-mirror")
			  "Other: "
			  ((input :name m :type text :name 60))
			  "(enter complete URL)"))))))
		(request-stream request))))))))
  
(defun cclan-redirector (request)
  (let* ((cookie (request-cookie request "CCLAN-SITE")))
    (cond (cookie
	   (request-redirect request
			     (merge-url (parse-urlstring cookie) 
					(request-unhandled-part request))))
	  (t
	   (request-redirect request "/cclan-choose-mirror")))))


