(in-package :telentweb)

(defvar *cliki-instance* nil)
(defvar *mccliki-instance* nil)
(defvar *alu-cliki-instance* nil)
(defvar *sbcl-cliki-instance* nil)
(defvar *buyitlocally-instance* nil)
(defvar *entomotomy-instance* nil)
(defvar *metacircles-instance* nil)
(defvar *toffeesix-instance* nil)

(defvar *listener* (make-instance 'serve-event-reverse-proxy-listener
				  :address #(0 0 0 0)
				  :port 8002))

(defun install-cliki (cliki-class  &rest rest
		      &key url-root &allow-other-keys)
  (let* ((url-root
	  (if (typep url-root 'url) url-root (parse-urlstring url-root)))
	 (c (apply #'make-instance cliki-class :url-root url-root rest))
	 (s (urlstring url-root)))
    (install-handler (http-listener-handler *listener*) c s nil)
    c))

(defclass telent-handler (dispatching-handler) 
  ((files :initarg :files :accessor telent-files)))

(defmethod handle-request-response ((h telent-handler) method request)
  (or (call-next-method)
      (handle-request (telent-files h) request)))

(defmethod shared-initialize :after ((root telent-handler) 
				     slot-names &rest args
				     &key pathname &allow-other-keys)
  (setf (telent-files root) 
	(make-instance 'static-file-handler  :pathname (pathname pathname)))
  (install-handler root (make-instance 
			 'redirect-handler :location
			 (parse-urlstring "http://localhost:25/"))
		   "cgi-bin/formmail" nil)
  (install-handler root (make-instance 'redirect-handler :location "/diary/")
		   "diary" t)
  (install-handler root
		   (make-instance 'diary-handler :data-directory
				  (merge-pathnames 
				   (make-pathname 
				    :directory
				    '(:relative "diary" "data")) pathname))
		   "diary/" nil)
  (install-handler root
		   (make-instance 'redirect-handler
				  :location
				  (parse-urlstring "http://www.cliki.net/"))
		   "cliki" nil)
  (install-handler root
		   (make-instance 'redirect-handler
				  :location
				  (parse-urlstring "http://sbcl-internals.cliki.net/"))
		   "sbcl-internals" nil)
  (install-handler root 'cclan-redirector "cclan/" nil)
  (install-handler root 'cclan-accept-redirect-cookie
		   "cclan-choose-mirror" nil))

  
(defun start ()
  "Install the Araneida event handlers (start answering requests).
Call this whenever you like"
  (setf (reverse-proxy-translations *listener*)
        (labels ((fwd-url (u)
                   (let ((nu (copy-url u))) (setf (url-port nu) 8002) nu)))
          (print (append
		    (loop for (u i) in (child-handlers (http-listener-handler *listener*))
			       collect (list u (urlstring (fwd-url (parse-urlstring u)))))
		     
		      '(("http://www.cliki.net/" "http://cliki.net:8002/")
			    ("http://cliki.net/" "http://www.cliki.net:8002/"))))))
	#+ (or)  (with-open-file (conf "/tmp/apache.telent.cf"
				              :direction :output
					             :if-does-not-exist :create)
				    (apache-conf-segment *listener* conf))
	(start-listening *listener*))


(defun stop ()
  "Remove the Araneida event handlers (stop answering requests).  Call
this whenever you like"
  (stop-listening *listener*))
